'use strict';

const tools = require('../tools.js');

function addInstance(data, options, callback) {
    let ip = data.Address;
    if (ip.includes('//')) {
        ip = ip.substr(ip.lastIndexOf('/')+1);
    }

    let instance = tools.findInstance(options, 'emby', obj => obj.native.ip === ip);

    if (!instance) {
        instance = {
            _id: tools.getNextInstanceID('emby', options),
            common: {
                name: 'emby',
                title: 'Emby (' + data.Name + ')'
            },
            native: {
                ip: ip,
                apiKey: '',
                deviceIds: '',
                timeout: 1500
            },
            comment: {
                add: data.Name + ' (' + ip + ')'
            }
        };
        options.newInstances.push(instance);
        callback(null, true, ip);
    } else {
        callback(null, false, ip);
    }
}

function detect(ip, device, options, callback) {
    tools.udpScan(ip, 7359, '0.0.0.0', 1234, 'who is EmbyServer?', 1400, (err, data) => {
        if (!err && data) {
            try {
                data = JSON.parse(data);
                if (data.Address){
                    addInstance(data, options, callback);
                }
            } catch (e) {
                options.log.error('emby error ' + e);
                callback(null, false, ip);
            }
        } else {
            err && options.log.error('emby error ' + err);
            callback(null, false, ip);
        }
    });
} // endDetect

exports.detect = detect;
exports.type = ['ip']; // TODO make to once and upd lookup
exports.timeout = 100;
